unit fTestMain;

interface

uses
  Windows, Classes, Graphics, Controls, Forms, ExtCtrls;

type

  TCellLayout = class;

{ TCell }
  TCell = class(TPersistent)
  private
    FColCount: Integer;
    FRowCount: Integer;
    function GetMergeCell(C, R, CW, RH: Single): TRect;
    procedure SetColCount(const Value: Integer);
    procedure SetRowCount(const Value: Integer);
  protected
    FParent: TCellLayout;
	  function CellWidth: Single;
    function CellHeight: Single;
    function CellFontHeight: Single;
    function CellRect(C, R: Single): TRect;
		function CellXY(C, R: Single): TPoint;
    function ClientRect: TRect;
    function ColRect(ACol: Single): TRect;
    function Height: Integer;
    function RowRect(ARow: Single): TRect;
    procedure Setup(ColCount, RowCount: Integer);
    function Width: Integer;
  public
    constructor Create(Parent: TCellLayout); reintroduce;
    procedure Draw;
    property 	ColCount:Integer read FColCount write SetColCount;
    property 	RowCount:Integer read FRowCount write SetRowCount;
    property 	MergeCell[C, R, CW, RH:Single]: TRect read GetMergeCell; default;
  end;

{ TCellLayout }  
  TCellLayout = class(TCustomControl)
  private
    FCell: TCell;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Cell: TCell read FCell;
  published
    property OnResize;
  end;

{ TfrmTestMain }  
  TfrmTestMain = class(TForm)
    procedure FormCreate(Sender: TObject);
  protected
    FLayout: TCellLayout;
    FPanels: array of TPanel;
    procedure DO_LAYOUT_REASIZE(Sender: TObject);
  end;

var
  frmTestMain: TfrmTestMain;

implementation

uses SysUtils, Math;

{$R *.dfm}

{ TCell }

constructor TCell.Create(Parent: TCellLayout);
begin
  inherited Create;
  FParent := Parent;
  Setup(40,25);
end;

function TCell.CellFontHeight: Single;
begin
  Result := Min(CellWidth, CellHeight);
end;

function TCell.CellHeight: Single;
begin
  Result := Height / FRowCount;
end;

function TCell.CellRect(C, R: Single): TRect;
begin
  Result := MergeCell[C, R, 1, 1];
end;

function TCell.CellWidth: Single;
begin
  Result := Width / FColCount;
end;

function TCell.CellXY(C, R: Single): TPoint;
begin
  Result := CellRect(C, R).TopLeft;
end;

function TCell.ClientRect: TRect;
begin
  Result := FParent.ClientRect;
  Dec(Result.Right);
  Dec(Result.Bottom);
end;

function TCell.ColRect(ACol: Single): TRect;
begin
  Result := MergeCell[ACol, 0, 1, FRowCount];
end;

function TCell.GetMergeCell(C, R, CW, RH: Single): TRect;
begin
  Result.Left 	:= Trunc(c*CellWidth+0.5);
  Result.Top 		:= Trunc(r*CellHeight+0.5);
  Result.Right	:= Trunc(Min(Width,(c+cw)*CellWidth)+0.5) + 1;
 	Result.Bottom	:= Trunc(Min(Height,(r+rh)*CellHeight)+0.5) + 1;
end;

function TCell.Height: Integer;
begin
  Result := FParent.ClientHeight - 1;
end;

function TCell.RowRect(ARow: Single): TRect;
begin
  Result := MergeCell[0, ARow, FColCount, 1];
end;

procedure TCell.SetColCount(const Value: Integer);
begin
  if Value <> FColCount then
  begin
    FColCount := Value;
    FParent.Invalidate;
  end;
end;

procedure TCell.SetRowCount(const Value: Integer);
begin
  if Value <> FRowCount then
  begin
    FRowCount := Value;
    FParent.Invalidate;
  end;
end;

function TCell.Width: Integer;
begin
  Result := FParent.ClientWidth - 1;
end;

procedure TCell.Setup(ColCount, RowCount: Integer);
begin
	if (FColCount<>ColCount) or (FRowCount<>RowCount) then 
  begin
		FColCount	:= ColCount;
  	FRowCount	:= RowCount;
  	FParent.Invalidate;
  end;
end;

procedure TCell.Draw;
var
  I: Integer;
begin
  with FParent.Canvas do
  begin
    Lock;
    try
      Pen.Color := clSilver;
      Pen.Width := 1;
      Pen.Style := psSolid;
      Pen.Mode := pmCopy;
      Brush.Color := clSilver;
      for I := 0 to ColCount-1 do
        FrameRect(ColRect(I));
      for I := 0 to RowCount-1 do
        FrameRect(RowRect(I));
    finally
      Unlock;
    end;
  end;
end;

{ TfrmTestMain }
procedure TfrmTestMain.DO_LAYOUT_REASIZE(Sender: TObject);
var
  I: Integer;
  r: TRect;
begin
  for I := 0 to Length(FPanels)-1 do
  begin
    case I of
      0: r := FLayout.Cell[00,00,40,20];
      1: r := FLayout.Cell[00,20,20,20];
      2: r := FLayout.Cell[00,40,60,20];
      3: r := FLayout.Cell[40,00,20,40];
      4: r := FLayout.Cell[60,00,20,10];
      5: r := FLayout.Cell[60,10,40,30];
    end;
    
    InflateRect(r, -2, -2);

    with FPanels[I], FLayout.Cell do
    begin
      Font.Height := Round(CellFontHeight)* 3;
      BoundsRect := r;
    end;

  end;
end;

procedure TfrmTestMain.FormCreate(Sender: TObject);
var
  I, PanelCount: Integer;

begin
  PanelCount := 6;
  FLayout := TCellLayout.Create(Self);
  SetLength(FPanels, PanelCount);
  for I := 0 to Length(FPanels)-1 do
  begin
    FPanels[I] := TPanel.Create(FLayout);
    FPanels[I].Caption := 'Panel' + IntToStr(I);
    FPanels[I].Font.Name := 'ARIAL';
    FPanels[I].Parent := FLayout;
  end;

  FLayout.Cell.Setup(100,50);
  FLayout.Align := alClient;
  FLayout.OnResize := DO_LAYOUT_REASIZE;
  
end;

{ TCellLayout }

constructor TCellLayout.Create(AOwner: TComponent);
begin
  inherited;
  FCell := TCell.Create(Self);
  Height := 100;
  Width := 100;
  Visible := True;
  Parent := TWinControl(AOwner);
  
end;

destructor TCellLayout.Destroy;
begin
  FreeAndNil(FCell);
  inherited;
end;

procedure TCellLayout.Paint;
begin
  inherited;
  if FCell <> nil then FCell.Draw;
end;

end.
